<?php
/**
 * XPW Framework
 *
 * LICENSE
 *
 * This source file is subject to the LGPLv3 license that is bundled
 * with this package in the file LICENSE.
 * It is also available through the world-wide-web at this URL:
 * http://xpw.ukrsoftsolutions.com/license
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@xpw.ukrsoftsolutions.com so we can send you a copy immediately.
 *
 * @category   XPW
 * @package    Xpw_Reflection_Class
 * @copyright  Copyright (c) 2008 UkrSoftSolutions (http://www.ukrsoftsolutions.com)
 * @version    1.0beta1
 * @license    http://xpw.ukrsoftsolutions.com/license     LGPLv3 License
 */

//include needed files.
require_once 'Xpw/Reflection/Helper/AttributeParser.php';
require_once 'Xpw/Reflection/ComplexProperty.php';
require_once 'Xpw/Reflection/Property.php';

/**
 * @category   XPW
 * @package    Xpw_Reflection_Class
 * @copyright  Copyright (c) 2008 UkrSoftSolutions (http://www.ukrsoftsolutions.com)
 * @license    http://xpw.ukrsoftsolutions.com/license     LGPLv3 License
 */
class Xpw_Reflection_Class extends ReflectionClass 
{
	/**
	 * Array of complex properties.
	 *
	 * @var Xpw_Reflection_ComplexProperty[]
	 */
	private $_complexProperties = array();
	
	/**
	 * Whether complex properties are parsed.
	 *
	 * @var bool
	 */
	private $_complexPropertiesParsed = false;
	
	/**
	 * Whether class has a complex property with the name specified.
	 *
	 * @param string $name
	 * @return bool
	 */
	public function hasComplexProperty($name)
	{
		$this->_parseComplexProperties();
		return in_array($name, $this->_complexPropertiesNames);
	}
	
	/**
	 * Returns complex properties of the class.
	 *
	 * @return Xpw_Reflection_ComplexProperty[]
	 */
	public function getComplexProperties()
	{
		$this->_parseComplexProperties();
		return array_values($this->_complexProperties);
	}
	
	/**
	 * Returns complex property reflection by the name.
	 *
	 * @param string $name
	 * @return Xpw_Reflection_ComplexProperty
	 */
	public function getComplexProperty($name)
	{
		$this->_parseComplexProperties();
		if (in_array($name, array_keys($this->_complexProperties)))
			return $this->_complexProperties[$name];
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . $this->getName() . '::' . $name . ' does not exist.');
		}
	}
	
	/**
	 * Parses methods of the class to find complex properties.
	 *
	 */
	private function _parseComplexProperties()
	{
		if (!$this->_complexPropertiesParsed)
		{
			$methods = parent::getMethods();
			foreach ($methods as $method)
			{
				$name = $method->getName();
				if (stripos($name, 'PropertyGet') !== false || stripos($name, 'PropertySet') !== false)
				{
					$complexPropertyName = substr($name, 0, -11);
					if (!in_array($complexPropertyName, array_keys($this->_complexProperties)))
					{
						$this->_complexProperties[$complexPropertyName] = new Xpw_Reflection_ComplexProperty($this->getName(), $complexPropertyName);
					}
				}
			}
			
			$this->_complexPropertiesParsed = true;
		}
	}
	
	/**
	 * Whether class has method with the name specified.
	 *
	 * @param string $name
	 * @return bool
	 */
	public function hasMethod($name)
	{
		if (stripos($name, 'PropertyGet') === false && 
			stripos($name, 'PropertySet') === false &&
			stripos($name, 'PropertyIsset') === false &&
			stripos($name, 'PropertyUnset') === false)
			return parent::hasMethod($name);
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Use hasComplexProperty method to get information about complex property.');
		}
	}
	
	/**
	 * Gets array of method reflections.
	 *
	 * @param int $filter
	 * @return Xpw_Reflection_Method[]
	 */
	public function getMethods($filter = null)
	{
		if (!is_int($filter))
			$filter = ReflectionMethod::IS_ABSTRACT | ReflectionMethod::IS_FINAL |
						ReflectionMethod::IS_PRIVATE | ReflectionMethod::IS_PROTECTED |
						ReflectionMethod::IS_PUBLIC | ReflectionMethod::IS_STATIC;
		$methods = parent::getMethods($filter);
		$result = array();
		foreach ($methods as $method)
		{
			$name = $method->getName();
			if (stripos($name, 'PropertyGet') === false && 
				stripos($name, 'PropertySet') === false &&
				stripos($name, 'PropertyIsset') === false &&
				stripos($name, 'PropertyUnset') === false)
			$result[] = new Xpw_Reflection_Method($this->getName(), $name);
		}
		return $result;
	}
	
	/**
	 * Gets method reflection by its name.
	 *
	 * @param string $name
	 * @return Xpw_Reflection_Method
	 */
	public function getMethod($name)
	{
		if ($this->hasMethod($name))
		{
			require_once 'Xpw/Reflection/Method.php';
			return new Xpw_Reflection_Method($this->getName(), $name);
		}
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Method ' . $this->getName() . '::' . $name . ' does not exist.');
		}
	}
	
	/**
	 * Gets property by name
	 *
	 * @param string $name
	 * @return Xpw_Reflection_Property
	 */
	public function getProperty($name)
	{
		try
		{
			 $property = parent::getProperty($name);
			 return new Xpw_Reflection_Property($this->getName(), $name);
		}
		catch (ReflectionException $e)
		{
			throw $e;	
		}
	}
	
	/**
	 * Gets array of class properties.
	 *
	 * @param int $filter
	 * @return Xpw_Reflection_Property[]
	 */
	public function getProperties($filter = null)
	{
		if (!is_int($filter))
			$filter = ReflectionProperty::IS_PRIVATE | ReflectionProperty::IS_PROTECTED | 
				ReflectionProperty::IS_PUBLIC | ReflectionProperty::IS_STATIC;
		$properties = parent::getProperties($filter);
		
		$result = array();
		
		foreach ($properties as $property)
		{
			$result[] = new Xpw_Reflection_Property($this->getName(), $property->getName());
		}
		
		return $result;
	}
	
	/**
	 * Returns string representation of the class instance.
	 *
	 * @return string
	 */
	public function __toString()
	{
		$result = $this->getDocComment() . "\n";
		$result .= 'Class [ ' . ($this->isUserDefined() ? '<user>' : '') . ($this->isInternal() ? '<internal>' : '');
		$result .= ' ' . ($this->isAbstract() ? 'abstract ' : ' ') . ($this->isInterface() ? 'interface ' : 'class');
		$result .= ' ' . $this->getName() . ' ' . (is_null($this->getParentClass()) ? '' : 'extends ' . $this->getParentClass()->getName());

		$interfaces = $this->getInterfaceNames();
		
		if (count($interfaces) > 0)
		{
			$result .= ' implements ';
			
			for ($i = 0; $i < count($interfaces); $i++)
			{
				$result .= $interfaces[$i];
				if ($i != count($interfaces) - 1)
					$result .= ', ';		
			}
		}
		
		$result .= " ] {\n";
		$result .= '  @@ ' . $this->getFileName() . ' ' . $this->getStartLine() . '-' . $this->getEndLine() . "\n\n";
		
		$constants = $this->getConstants();
		
		$result .= '  - Constants [' . count($constants) . "] {\n";
		
		foreach ($constants as $key => $value)
		
			$result .= '    Constant [ ' . gettype($value) . ' ' . $key . ' ] { ' . $value . " }\n"; 
		
		$result .= "  }\n\n";
		
		$staticProperties = $this->getProperties(ReflectionProperty::IS_STATIC);
		
		$result .= '  - Static properties [' . count($staticProperties) . "] {\n";
		
		foreach ($staticProperties as $staticProperty)
			$result .= '    ' . $staticProperty;

		$result .= "  }\n\n";
		
		$staticMethods = $this->getMethods(ReflectionMethod::IS_STATIC);
		
		$result .= '  - Static methods [' . count($staticMethods) . "] {\n";
		
		foreach ($staticMethods as $staticMethod)
		{
			$str = new Xpw_System_String($staticMethod);
			$result .= $str->indentMultiline('    ');
		}
		
		$result .= "  }\n\n";
		
		$properties = $this->getProperties();
		
		$result .= '  - Properties [' . count($properties) . "] {\n";
		foreach ($properties as $property)
		{
			if (!$property->isStatic())
				$result .= '    ' . $property;
		}
			
		$result .= "  }\n\n";
		
		$complexProperties = $this->getComplexProperties();
		
		$result .= '  - Complex properties [' . count($complexProperties) . "] {\n";
		
		foreach ($complexProperties as $complexProperty)
		{
			$str = new Xpw_System_String($complexProperty);
			$result .= $str->indentMultiline('    ') . "\n";
		}
		
		$methods = $this->getMethods();
		
		$result .= ' - Methods [' . count($methods) . "] {\n";
		
		foreach ($methods as $method)
		{
			if (!$method->isStatic())
			{
				$str = new Xpw_System_String($method);
				$result .= $str->indentMultiline('    ') . "\n";
			}
		}
		
		$result .= "  }\n";
		
		$result .= "}\n";
		
		return $result;
	}
}
?>