<?php
/**
 * XPW Framework
 *
 * LICENSE
 *
 * This source file is subject to the LGPLv3 license that is bundled
 * with this package in the file LICENSE.
 * It is also available through the world-wide-web at this URL:
 * http://xpw.ukrsoftsolutions.com/license
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@xpw.ukrsoftsolutions.com so we can send you a copy immediately.
 *
 * @category   XPW
 * @package    Xpw_Reflection_ComplexProperty
 * @copyright  Copyright (c) 2008 UkrSoftSolutions (http://www.ukrsoftsolutions.com)
 * @version    1.0beta1
 * @license    http://xpw.ukrsoftsolutions.com/license     LGPLv3 License
 */

//include needed files.
require_once 'Xpw/Reflection/Method.php';
require_once 'Xpw/System/String.php';

/**
 * @category   XPW
 * @package    Xpw_Reflection_ComplexProperty
 * @copyright  Copyright (c) 2008 UkrSoftSolutions (http://www.ukrsoftsolutions.com)
 * @license    http://xpw.ukrsoftsolutions.com/license     LGPLv3 License
 */
class Xpw_Reflection_ComplexProperty extends Xpw_System_Base implements Reflector 
{
	/**
	 * Class being reflected.
	 *
	 * @var string
	 */
	protected $_class;
	
	/**
	 * Complex property being reflected.
	 *
	 * @var string
	 */
	protected $_name;
	
	/**
	 * Reflection of the getter method.
	 *
	 * @var Xpw_Reflection_Method
	 */
	protected $_getterReflection;
	
	/**
	 * Reflection of the setter method.
	 *
	 * @var Xpw_Reflection_Method
	 */
	protected $_setterReflection;
	
	/**
	 * Reflection of the isset method.
	 *
	 * @var Xpw_Reflection_Method
	 */
	protected $_issetReflection;
	
	/**
	 * Reflection of the unset method.
	 *
	 * @var Xpw_Reflection_Method
	 */
	protected $_unsetReflection;
	
	/**
	 * Initializes new instance.
	 *
	 * @param string $class
	 * @param string $name
	 */
	public function __construct($class, $name)
	{
		if (!class_exists($class))
			throw new Xpw_Reflection_Exception('Class ' . $class . ' does not exist.');
		$this->_class = $class;
		$this->_name = $name;
		
		$getter = method_exists($class, $name . 'PropertyGet');
		$setter = method_exists($class, $name . 'PropertySet');
		$isset = method_exists($class, $name . 'PropertyIsset');
		$unset = method_exists($class, $name . 'PropertyUnset');
		
		if ($getter)
			$this->_getterReflection = new Xpw_Reflection_Method($class, $name . 'PropertyGet');
		if ($setter)
			$this->_setterReflection = new Xpw_Reflection_Method($class, $name . 'PropertySet');
		if ($isset)
			$this->_issetReflection = new Xpw_Reflection_Method($class, $name . 'PropertyIsset');
		if ($unset)
			$this->_unsetReflection = new Xpw_Reflection_Method($class, $name . 'PropertyUnset');
		if (!$setter && !$getter)
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex Property ' . $class . '::' . $name . ' does not exist.');
		} 
	}
	
	/**
	 * Whether Complex Property has setter.
	 *
	 * @return bool
	 */
	public function hasSetter()
	{
		return isset($this->_setterReflection);
	}
	
	/**
	 * Wheter Complex Property has getter.
	 *
	 * @return bool
	 */
	public function hasGetter()
	{
		return isset($this->_getterReflection);
	}
	
	/**
	 * Whether Complex Property has isset checker.
	 *
	 * @return bool
	 */
	public function hasIsset()
	{
		return isset($this->_issetReflection);
	}
	
	/**
	 * Whether Complex Property can be used as parameter of unset() function.
	 *
	 * @return bool
	 */
	public function hasUnset()
	{
		return isset($this->_unsetReflection);
	}
	
	/**
	 * Gets value of the complex property.
	 *
	 * @param Xpw_System_Base $object
	 * @return mixed
	 */
	public function getValue($object)
	{
		if ($this->hasGetter() && $this->_getterReflection->isPublic())
			return $this->_getterReflection->invokeArgs($object, array());
		elseif (!$this->hasGetter())
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' getter does not exist.');
		}
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' is not public.');
		}
	}
	
	/**
	 * Sets value of the complex property.
	 *
	 * @param Xpw_System_Base $object
	 * @param mixed $value
	 */
	public function setValue($object, $value)
	{
		if ($this->hasSetter() && $this->_setterReflection->isPublic())
			$this->_setterReflection->invokeArgs($object, array($value));
		elseif (!$this->hasSetter())
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' setter does not exist.');
		}
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' is not public.');
		}
	}
	
	/**
	 * Whether complex property is set.
	 *
	 * @param Xpw_System_Base $object
	 * @return bool
	 */
	public function issetValue($object)
	{
		if ($this->hasIsset()&& $this->_issetReflection->isPublic())
			return $this->_issetReflection->invokeArgs($object, array());
		elseif (!$this->hasIsset())
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' isset does not exist.');
		}
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' isset is not public.');
		}
	}
	
	/**
	 * Unsets complex property.
	 *
	 * @param Xpw_System_Base $object
	 */
	public function unsetValue($object)
	{
		if ($this->hasUnset()&& $this->_unsetReflection->isPublic())
			$this->_unsetReflection->invokeArgs($object, array());
		elseif (!$this->hasUnset())
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' unset does not exist.');
		}
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' unset is not public.');
		}
	}
	
	/**
	 * Whether getter is public.
	 *
	 * @return bool
	 */
	public function isGetterPublic()
	{
		if ($this->hasGetter())
			return $this->_getterReflection->isPublic();
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' getter does not exist.');
		}
	}
	
	/**
	 * Whether setter is public.
	 *
	 * @return bool
	 */
	public function isSetterPublic()
	{
		if ($this->hasSetter())
			return $this->_setterReflection->isPublic();
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' setter does not exist.');
		}
	}
	
	/**
	 * Wheter isset method of the complex property is public.
	 *
	 * @return bool
	 */
	public function isIssetPublic()
	{
		if ($this->hasIsset())
			return $this->_issetReflection->isPublic();
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' isset does not exist.');
		}
	}
	
	/**
	 * Whether unset method of the complex property is public.
	 *
	 * @return bool
	 */
	public function isUnsetPublic()
	{
		if ($this->hasUnset())
			return $this->_unsetReflection->isPublic();
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' unset does not exist.');
		}
	}
	
	/**
	 * Whether getter is private.
	 *
	 * @return bool
	 */
	public function isGetterPrivate()
	{
		if ($this->hasGetter())
			return $this->_getterReflection->isPrivate();
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' getter does not exist.');
		}
	}
	
	/**
	 * Whether setter is private.
	 *
	 * @return bool
	 */
	public function isSetterPrivate()
	{
		if ($this->hasSetter())
			return $this->_setterReflection->isPrivate();
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' setter does not exist.');
		}
	}
	
	/**
	 * Wheter isset method of the complex property is private.
	 *
	 * @return bool
	 */
	public function isIssetPrivate()
	{
		if ($this->hasIsset())
			return $this->_issetReflection->isPrivate();
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' isset does not exist.');
		}
	}
	
	/**
	 * Whether unset method of the complex property is private.
	 *
	 * @return bool
	 */
	public function isUnsetPrivate()
	{
		if ($this->hasUnset())
			return $this->_unsetReflection->isPrivate();
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' unset does not exist.');
		}
	}
	
	/**
	 * Whether getter is protected.
	 *
	 * @return bool
	 */
	public function isGetterProtected()
	{
		if ($this->hasGetter())
			return $this->_getterReflection->isProtected();
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' getter does not exist.');
		}
	}
	
	/**
	 * Whether setter is protected.
	 *
	 * @return bool
	 */
	public function isSetterProtected()
	{
		if ($this->hasSetter())
			return $this->_setterReflection->isProtected();
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' setter does not exist.');
		}
	}
	
	/**
	 * Wheter isset method of the complex property is protected.
	 *
	 * @return bool
	 */
	public function isIssetProtected()
	{
		if ($this->hasIsset())
			return $this->_issetReflection->isProtected();
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' isset does not exist.');
		}
	}
	
	/**
	 * Whether unset method of the complex property is protected.
	 *
	 * @return bool
	 */
	public function isUnsetProtected()
	{
		if ($this->hasUnset())
			return $this->_unsetReflection->isProtected();
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' unset does not exist.');
		}
	}
	
	/**
	 * Whether complex property getter is static.
	 * 
	 * Complex properties
	 * are accessible in object context only so
	 * false is always returned. 
	 *
	 * @return bool
	 */
	public function isGetterStatic()
	{
		return false;
	}
	
	/**
	 * Whether complex property setter is static.
	 * 
	 * Complex property are accessible in object context only so
	 * false is always returned. 
	 *
	 * @return bool
	 */
	public function isSetterStatic()
	{
		return false;
	}
	
	/**
	 * Whether isset method of the complex property is static.
	 * 
	 * Complex property are accessible in object context only so
	 * false is always returned. 
	 *
	 * @return bool
	 */
	public function isIssetStatic()
	{
		return false;
	}
	
	/**
	 * Whether unset method of the complex property is static.
	 * 
	 * Complex property are accessible in object context only so
	 * false is always returned. 
	 *
	 * @return bool
	 */
	public function isUnsetStatic()
	{
		return false;
	}
	
	/**
	 * Returns reflection of the getter. 
	 *
	 * @return ReflectionMethod
	 */
	public function getGetterReflection()
	{
		if ($this->hasGetter())
			return $this->_getterReflection;
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' getter does not exist.');
		}
	}
	
	/**
	 * Returns reflection of the setter.
	 *
	 * @return ReflectionMethod
	 */
	public function getSetterReflection()
	{
		if ($this->hasSetter())
			return $this->_setterReflection;
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Complex property ' . 
				$this->_class . '::' . $this->_name . ' setter does not exist.');
		}
	}
	
	/**
	 * Returns reflection of the isset method of the complex property.
	 *
	 * @return ReflectionMethod
	 */
	public function getIssetReflection()
	{
		if ($this->hasIsset())
			return $this->_issetReflection;
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Isset method of the complex property ' . 
				$this->_class . '::' . $this->_name . ' does not exist.');
		}
	}
	
	/**
	 * Returns reflection of the unset method of the complex property.
	 *
	 * @return ReflectionMethod
	 */
	public function getUnsetReflection()
	{
		if ($this->hasUnset())
			return $this->_unsetReflection;
		else
		{
			require_once 'Xpw/Reflection/Exception.php';
			throw new Xpw_Reflection_Exception('Unset method of the complex property ' . 
				$this->_class . '::' . $this->_name . ' does not exist.');
		}
	}
	
	/**
	 * Returns declaring class.
	 *
	 * @return Xpw_Reflection_Class
	 */
	public function getDeclaringClass()
	{
		return new Xpw_Reflection_Class($this->_class);
	}
	
	/**
	 * Returns name of the complex property.
	 *
	 * @return string
	 */
	public function getName()
	{
		return $this->_name;
	}
	
	/**
	 * Export method of the Reflector interface.
	 *
	 * @param string $class
	 * @param string $name
	 * @return Xpw_Reflection_ComplexProperty
	 */
	public static function export($class, $name)
	{
		return new Xpw_Reflection_ComplexProperty($class, $name);
	}
	
	/**
	 * Returns string representation of the object.
	 *
	 * @return string
	 */
	public function __toString()
	{
		$result = 'Complex Property [ '. $this->getName() . " ] {\n".
				"\n";
		if ($this->hasGetter())
		{
			$string = new Xpw_System_String('Getter ' . $this->_getterReflection);
			$result .= $string->indentMultiline('  ');
		}
		if ($this->hasSetter())
		{
			$string = new Xpw_System_String('Setter ' . $this->_setterReflection);
			$result .= $string->indentMultiline('  ');
		}
		if ($this->hasIsset())
		{
			$string = new Xpw_System_String('Isset ' . $this->_issetReflection);
			$result .= $string->indentMultiline('  ');
		}
		if ($this->hasIsset())
		{
			$string = new Xpw_System_String('Isset ' . $this->_issetReflection);
			$result .= $string->indentMultiline('  ');
		}
		$result .= "}\n";
		return $result;
	}
	
	/**
	 * Instances of this class cannot be cloned.
	 *
	 */
	final private function __clone()
	{
	}
}
?>